@@ -5,36 +5,28 @@ from gantry.logger.main import LOGGING_LEVEL_T
 from gantry.logger.main import _init as logger_init
 from gantry.logger.main import (
     get_client,
-    instrument,
-    log_feedback,
-    log_feedback_event,
     log_file,
-    log_prediction_event,
-    log_predictions,
     log_record,
     log_records,
     ping,
     ready,
     setup_logger,
 )
+from gantry.logger.utils import JoinKey
 from gantry.query.main import _init as query_init
 from gantry.utils import compute_feedback_id
 
 __all__ = [
     "init",
-    "instrument",
     "log_record",
     "log_records",
     "log_file",
     "ping",
     "ready",
-    "log_feedback",
-    "log_predictions",
-    "log_feedback_event",
-    "log_prediction_event",
     "get_client",
     "setup_logger",
     "compute_feedback_id",
+    "JoinKey",
 ]
 
 
