@@ -2,6 +2,7 @@ import logging
 
 import click
 
+from gantry.cli.application import application
 from gantry.cli.projection import projection
 
 
@@ -31,6 +32,7 @@ def cli(ctx, loglevel):
 
 
 cli.add_command(projection)
+cli.add_command(application)
 
 if __name__ == "__main__":
     cli()
