@@ -1,9 +1,8 @@
 import logging
 import uuid
+import pathlib
 from typing import Any, Dict, List, Optional
-
 from typeguard import typechecked
-
 from gantry.api_client import APIClient
 from gantry.dataset.gantry_dataset import GantryDataset
 from gantry.exceptions import DatasetNotFoundError, GantryRequestException
@@ -18,7 +17,7 @@ class GantryDatasetClient:
         email: str,
         bucket_name: Optional[str] = None,
         aws_region: Optional[str] = None,
-        workspace: str = ".",
+        workspace: str = str(pathlib.Path().absolute()),
     ):
         self._api_client = api_client
         self.email = email
