@@ -6,6 +6,9 @@ import logging
 import os
 import shutil
 import uuid
+import yaml
+import jinja2
+import datasets
 from collections import defaultdict
 from dataclasses import asdict, dataclass
 from pathlib import Path
@@ -19,14 +22,16 @@ from gantry.dataset import constants
 from gantry.exceptions import (
     DatasetCommitNotFoundError,
     DatasetDeletedException,
-    GantryRequestException,
+    DatasetHeadOutOfDateException,
     DatasetNotFoundError,
+    GantryRequestException,
 )
 from gantry.utils import (
     download_file_from_url,
     get_files_checksum,
     parse_s3_path,
     upload_file_to_url,
+    list_all_files,
 )
 
 logger = logging.getLogger(__name__)
@@ -57,9 +62,9 @@ def _ensure_not_deleted(func: Callable) -> Callable:
     return wrapper
 
 
-def _ensure_dataset_exist_locally(func: Callable) -> Callable:
+def _ensure_dataset_exists_locally(func: Callable) -> Callable:
     """
-    Decorator to ensure dataset exist locally, if not will warn the user to download the dataset.
+    Decorator to ensure dataset exists locally, if not will warn the user to download the dataset.
     """
 
     @functools.wraps(func)
@@ -125,7 +130,7 @@ class GantryDataset:
         )
         return response["data"]
 
-    @_ensure_dataset_exist_locally
+    @_ensure_dataset_exists_locally
     def get_diff(self) -> Dict[str, List[str]]:
         """
         Get local changes which hasn't been committed
@@ -145,9 +150,9 @@ class GantryDataset:
         }
 
     @typechecked
-    @_ensure_dataset_exist_locally
+    @_ensure_dataset_exists_locally
     @_ensure_not_deleted
-    def create_commit(self, message: str) -> Dict[str, Any]:
+    def commit(self, message: str) -> Dict[str, Any]:
         """
         This method will compute the difference between the current commit and the local directory,
         upload new files and modified files to dataset repo. After that we will update
@@ -169,6 +174,19 @@ class GantryDataset:
         ):
             logger.warning("No local changes to commit!")
             return self._get_current_commit()
+
+        # Check that HEAD is up to date
+        resp = self._api_client.request(
+            "GET", f"/api/v1/datasets/{self._dataset_id}/commits", raise_for_status=True
+        )
+        latest_commit_id = resp["data"][0]["id"]
+
+        with open(self._get_file_path(constants.DATASET_HEAD_FILE), "r") as f:
+            current_commit = json.load(f)
+
+            if current_commit["id"] != latest_commit_id:
+                raise DatasetHeadOutOfDateException
+
         diff[constants.NEW_FILES] = self._upload_files(diff[constants.NEW_FILES])
         diff[constants.MODIFIED_FILES] = self._upload_files(diff[constants.MODIFIED_FILES])
 
@@ -202,6 +220,12 @@ class GantryDataset:
             )
         except GantryRequestException as e:
             self._recover_file(constants.DATASET_MANIFEST_FILE)
+            if (
+                e.status_code == 400
+                and 'Details: {"error":"Parent commit out of date!","response":"error"}'
+                in e.args[0]
+            ):
+                raise DatasetHeadOutOfDateException
             raise e
 
         commit_info = resp["data"]
@@ -337,13 +361,13 @@ class GantryDataset:
         )
 
         commit_info = resp["data"]
-        self.sync(commit_info["id"])
+        self.pull(commit_info["id"])
 
         return commit_info
 
     @typechecked
     @_ensure_not_deleted
-    def sync(self, commit_id: Optional[Union[str, uuid.UUID]] = None) -> Dict[str, Any]:
+    def pull(self, commit_id: Optional[Union[str, uuid.UUID]] = None) -> Dict[str, Any]:
         """
         Sync local dataset folder based on a commit id. If commit_id was not provided, will sync
         based on the latest commit.
@@ -503,14 +527,14 @@ class GantryDataset:
                 file_info = json.loads(line)
                 commit_snapshot[file_info[constants.FILE_NAME]] = file_info
 
-        # don't need url for new files
+        # don't need url for new files and use checksum for empty string
         sync_diff[constants.NEW_FILES] = [
             DatasetFileInfo(
                 file_name=file_name,
                 url="",
-                sha256=checksum,
+                sha256=constants.EMPTY_STR_SHA256,
             )
-            for file_name, checksum in repo_diff[constants.NEW_FILES]
+            for file_name, _ in repo_diff[constants.NEW_FILES]
         ]
         # retrieve url for modified files
         sync_diff[constants.MODIFIED_FILES] = [
@@ -573,7 +597,7 @@ class GantryDataset:
 
         for file_name in commit_snapshot:
             if file_name not in local_files:
-                repo_diff[constants.DELETED_FILES].append((file_name, checksum))
+                repo_diff[constants.DELETED_FILES].append((file_name, constants.EMPTY_STR_SHA256))
 
         return repo_diff
 
@@ -700,3 +724,69 @@ class GantryDataset:
                 logger.info(f"successfully upload: {item.file_name}")
 
         return file_list
+
+    def _extract_feature_type(self) -> Dict[str, Any]:
+        """
+        Read feature type from dataset_config and convert them to huggingface feature type
+        Returns:
+            A key value pair of Dict where key is the column name and value is huggingface
+            feature type
+        """
+        columns = dict()
+        config = yaml.safe_load(
+            (Path(self.workspace) / self.dataset_name / constants.DATASET_CONFIG_FILE).open(
+                mode="r"
+            )
+        )
+
+        if constants.DATASET_FEATURES_KEY in config:
+            for feature_name, feature_dtype in config[constants.DATASET_FEATURES_KEY].items():
+                columns[feature_name] = constants.GANTRY_2_HF_DTYPE[feature_dtype]
+
+        if constants.DATASET_FEEDBACK_KEY in config:
+            for feature_name, feature_dtype in config[constants.DATASET_FEEDBACK_KEY].items():
+                columns[feature_name] = constants.GANTRY_2_HF_DTYPE[feature_dtype]
+
+        return columns
+
+    def get_huggingface_dataset(self):
+        """
+        Create a huggingface dataset based on the local dataset files
+
+        This method will create a huggingface load script based on the dataset config and
+        then load the dataset into huggingface pyarrow dataset.
+        """
+
+        env = jinja2.Environment(
+            loader=jinja2.FileSystemLoader(
+                os.path.join(os.path.dirname(os.path.realpath(__file__)), "templates")
+            )
+        )
+
+        template = env.get_template("load_script_template.py.jinja")
+
+        training_files = [
+            str(p)
+            for p in list_all_files(
+                Path(self.workspace) / self.dataset_name / constants.TABULAR_MANIFESTS
+            )
+        ]
+
+        content = template.render(
+            training_file_list=training_files,
+            dataset_version="1.0.0",  # Format should be x.y.z with {{x,y,z}} being digits
+            dataset_name=self.dataset_name,
+            features=self._extract_feature_type(),
+        )
+
+        hf_loading_script = (
+            Path(self.workspace)
+            / self.dataset_name
+            / constants.HF_FOLDER
+            / f"{self.dataset_name}.py"
+        )
+        os.makedirs(hf_loading_script.parent, exist_ok=True)
+        with hf_loading_script.open("w") as f:
+            f.write(content)
+
+        return datasets.load_dataset(str(hf_loading_script))
