@@ -289,7 +289,7 @@ class Gantry:
         sample_rate: float = 1.0,
         as_batch: bool = False,
         tags: Optional[Dict[str, str]] = None,
-        join_keys: Optional[List[str]] = None,
+        join_keys: Optional[Union[List[str], pd.Series]] = None,
     ) -> Tuple[Optional[str], List[str]]:
         """
         Function to record a batch of events containing predictions (inputs and outputs),
@@ -358,15 +358,13 @@ class Gantry:
                 data. E.g. you can specify which environment the data belongs to by setting "env"
                 tag like this tags = {"env": "environment1"} if not assigned we will use Gantry
                 client's environment value as the default environment.
-            join_keys  (optional, List[str]): provide keys to identify each record.
-                These keys can be used later to provide feedback. If not provided,
+            join_keys  (optional, Union[List[str], pd.Series[str]]): provide keys to identify
+                each record. These keys can be used later to provide feedback. If not provided,
                 a random record key will be generated for each record.
 
         Returns:
-            A tuple of:
-                (str) batch_id: The id of the logged batch. Returns None if the records are not to
-                    be logged as a batch
-                (list[str]) join_keys: The record keys
+            A tuple of (batch_id, list[join_keys]): The batch_id will be None if records are not
+                logged as batch. The list of join_keys will be the records keys.
 
         """
         if inputs is not None and isinstance(inputs, pd.Series):
@@ -375,6 +373,9 @@ class Gantry:
             outputs = outputs.to_frame()
         if feedbacks is not None and isinstance(feedbacks, pd.Series):
             feedbacks = feedbacks.to_frame()
+        if join_keys is not None and isinstance(join_keys, pd.Series):
+            join_keys = join_keys.to_list()
+            check_type("join_keys", join_keys, List[str])
 
         some_preds_exist = _is_not_empty(inputs) or _is_not_empty(outputs)
         preds_exist = _is_not_empty(inputs) and _is_not_empty(outputs)
