@@ -201,7 +201,7 @@ def log_records(
     sample_rate: float = 1.0,
     as_batch: bool = False,
     tags: Optional[Dict[str, str]] = None,
-    join_keys: Optional[List[str]] = None,
+    join_keys: Optional[Union[List[str], pd.Series]] = None,
 ) -> None:
     return _CLIENT.log_records(**locals())  # type: ignore[union-attr]
 
