@@ -206,23 +206,32 @@ def upload_file_to_url(presigned_url: str, local_path: Path):
     return res.headers["x-amz-version-id"]
 
 
-def list_all_files(dirpath):
+def is_hidden(path):
+    return str(path.name).startswith(".")
+
+
+def list_all_files(dirpath, include_hidden_file=False):
     """
     List all files in a folder
 
     Args:
-        dirpath (Path): _description_
-
+        dirpath (Path): target directory
+        include_hidden_file (bool): whether include hidden file or not
     Returns:
         List[Path]
     """
     assert dirpath.is_dir()
     file_list = []
+
+    if not include_hidden_file and is_hidden(dirpath):
+        return file_list
+
     for x in dirpath.iterdir():
         if x.is_file():
-            file_list.append(x)
+            if include_hidden_file or not is_hidden(x):
+                file_list.append(x)
         elif x.is_dir():
-            file_list.extend(list_all_files(x))
+            file_list.extend(list_all_files(x, include_hidden_file))
 
     return file_list
 
@@ -249,7 +258,7 @@ def get_files_checksum(path):
     Returns:
         {file name: sha256 checksum}
     """
-
+    # Do not include hidden files to avoid adding .DS_Store or other system files by accident
     file_list = list_all_files(path)
     return dict(
         [(str(f.relative_to(path)), get_s3_sha256_checksum(f.read_bytes())) for f in file_list]
