@@ -5,6 +5,7 @@ from pathlib import Path
 import pandas as pd
 import pytest
 import responses
+from datasets import Value
 from mock import patch
 from responses import matchers
 
@@ -14,6 +15,7 @@ from gantry.dataset.constants import DELETED_FILES, MODIFIED_FILES, NEW_FILES
 from gantry.dataset.gantry_dataset import DatasetFileInfo, GantryDataset
 from gantry.exceptions import (
     DatasetCommitNotFoundError,
+    DatasetHeadOutOfDateException,
     GantryException,
     GantryRequestException,
     DatasetNotFoundError,
@@ -160,7 +162,32 @@ def test_create_commit_without_download(gantry_dataset_obj):
     # overwrite the dataset name
     gantry_dataset_obj.dataset_name = "dataset_not_exist"
     with pytest.raises(DatasetNotFoundError):
-        gantry_dataset_obj.create_commit("test commit")
+        gantry_dataset_obj.commit("test commit")
+
+
+def test_get_huggingface_dataset(gantry_dataset_obj):
+    """
+    Test get local edit diff
+    """
+    # overwrite the dataset name
+    gantry_dataset_obj.dataset_name = "hf_test"
+
+    hf_dataset = gantry_dataset_obj.get_huggingface_dataset()
+
+    assert len(hf_dataset["train"]) == 40
+    assert hf_dataset["train"].features == {
+        "inputs.credit_history": Value(dtype="string", id=None),
+        "inputs.description": Value(dtype="string", id=None),
+        "inputs.image_uri": Value(dtype="string", id=None),
+        "inputs.income": Value(dtype="int64", id=None),
+        "inputs.loan_id": Value(dtype="string", id=None),
+        "inputs.prediction": Value(dtype="bool", id=None),
+        "inputs.timestamp": Value(dtype="timestamp[ns, tz=UTC]", id=None),
+        "outputs.output": Value(dtype="float64", id=None),
+        "feedback.loan_repaid": Value(dtype="bool", id=None),
+        "__time": Value(dtype="timestamp[ns, tz=UTC]", id=None),
+    }
+    assert hf_dataset["train"].builder_name == "hf_test"
 
 
 def setup_resp_for_deleted_check(resp, is_deleted=False, dataset_name=DATASET_NAME):
@@ -181,9 +208,22 @@ def setup_resp_for_deleted_check(resp, is_deleted=False, dataset_name=DATASET_NA
     )
 
 
+def setup_resp_for_head_check(resp, latest_commit, dataset_id=DATASET_ID):
+    resp.add(
+        resp.GET,
+        f"{HOST}/api/v1/datasets/{dataset_id}/commits",
+        json={
+            "response": "ok",
+            "data": [latest_commit],
+        },
+        headers={"Content-Type": "application/json"},
+    )
+
+
 def setup_resp_for_create_commit(
     datadir, resp, create_commit_resp_json, commit_id, create_commit_status_code=200
 ):
+    # fake urls for uploading files
     resp.add(
         resp.POST,
         f"{HOST}/api/v1/datasets/{DATASET_ID}/presign/putobject",
@@ -199,7 +239,6 @@ def setup_resp_for_create_commit(
         },
         headers={"Content-Type": "application/json"},
     )
-
     resp.add(
         resp.PUT,
         f"{HOST}/fake_presigned_url",
@@ -210,6 +249,7 @@ def setup_resp_for_create_commit(
     with open(datadir / f"{DATASET_NAME}/.dataset_metadata/HEAD") as f:
         old_commit = json.load(f)
 
+    # for the actual commit creation request
     resp.add(
         resp.POST,
         f"{HOST}/api/v1/datasets/{DATASET_ID}/commits",
@@ -230,6 +270,102 @@ def setup_resp_for_create_commit(
     )
 
 
+def setup_resp_for_sync(
+    resp,
+    commit_id,
+    commit_json,
+    manifest_file,
+    dataset_id=DATASET_ID,
+    do_fake_presigned=True,
+):
+    # getting commit info
+    if commit_id is not None:  # if commit_id is specified
+        resp.add(
+            resp.GET,
+            f"{HOST}/api/v1/datasets/{dataset_id}/commits/{commit_id}",
+            json={
+                "response": "ok",
+                "data": commit_json,
+            },
+            headers={"Content-Type": "application/json"},
+        )
+    else:  # if we're syncing to latest
+        resp.add(
+            resp.GET,
+            f"{HOST}/api/v1/datasets/{dataset_id}/commits",
+            json={
+                "response": "ok",
+                "data": [commit_json],
+            },
+            headers={"Content-Type": "application/json"},
+        )
+
+    # getting the fake urls for the objects
+    resp.add(
+        resp.POST,
+        f"{HOST}/api/v1/datasets/{dataset_id}/presign/getobject",
+        json={
+            "response": "ok",
+            "data": {
+                MANIFEST_OBJ_KEY: f"{HOST}/fake_manifest",
+                CONF_OBJ_KEY: f"{HOST}/fake_presigned_url",
+                README_OBJ_KEY: f"{HOST}/fake_presigned_url",
+            },
+        },
+        headers={"Content-Type": "application/json"},
+    )
+
+    # adding responses for the fake download urls
+    req_kwargs = {
+        "stream": True,
+    }
+    resp.add(
+        resp.GET,
+        f"{HOST}/fake_manifest",
+        body=manifest_file.read(),
+        match=[matchers.request_kwargs_matcher(req_kwargs)],
+    )
+    if do_fake_presigned:
+        resp.add(
+            resp.GET,
+            f"{HOST}/fake_presigned_url",
+            body="fake_response",
+            match=[matchers.request_kwargs_matcher(req_kwargs)],
+        )
+
+
+def setup_resp_for_add_file(resp, commit_message, filename, file):
+    resp.add(
+        resp.POST,
+        f"{HOST}/api/v1/datasets/{DATASET_ID}/file",
+        json={
+            "response": "ok",
+            "data": {
+                "created_at": "Wed, 05 Oct 2022 05:42:13 GMT",
+                "created_by": "db459d6d-c83b-496d-b659-e48bca971156",
+                "dataset_id": DATASET_ID,
+                "id": "8ee0f6d5-c84c-473a-931a-5148b2e704d7",
+                "is_latest_commit": True,
+                "message": commit_message,
+                "metadata_s3_file_version": MANIFEST_VERSION_ID,
+                "parent_commit": "eb2e7242-3340-4edf-8366-90d4fce897ce",
+            },
+        },
+        headers={"Content-Type": "application/json"},
+        match=[
+            matchers.multipart_matcher(
+                {
+                    "file": (filename, file),
+                },
+                data={
+                    "email": "test@gantry.dev",
+                    "commit_msg": commit_message,
+                },
+            )
+        ],
+    )
+
+
 @patch("uuid.uuid4", lambda: "test-uuid-1")
 def test_create_commit(datadir, gantry_dataset_obj, commit_history):
     """
@@ -240,8 +376,10 @@ def test_create_commit(datadir, gantry_dataset_obj, commit_history):
             datadir, resp, {"response": "ok", "data": commit_history[0]}, "test-uuid-1"
         )
         setup_resp_for_deleted_check(resp)
+        with open(datadir / f"{DATASET_NAME}/.dataset_metadata/HEAD") as f:
+            setup_resp_for_head_check(resp, json.load(f))  # we are on latest commit
 
-        gantry_dataset_obj.create_commit(COMMIT_MSG)
+        gantry_dataset_obj.commit(COMMIT_MSG)
 
         diff = gantry_dataset_obj.get_diff()
 
@@ -255,24 +393,18 @@ def test_create_commit(datadir, gantry_dataset_obj, commit_history):
 
 
 @patch("uuid.uuid4", lambda: "test-uuid-1")
-def test_create_commit_failure(datadir, gantry_dataset_obj):
+def test_create_commit_failure(datadir, gantry_dataset_obj, commit_history):
     """
-    Test create commit failure, in this case all upload will finish successfully but the create
-    commit call will fail. This will happen during a race condition when another user committed a
-    change and the local copy out of date.
+    Test create commit failure, in this case we detect the head is not up to date and stop
+    prematurely. This will happen during a race condition when another user committed a change and
+    the local copy has not been synced subsequently.
     """
     with responses.RequestsMock() as resp:
-        setup_resp_for_create_commit(
-            datadir,
-            resp,
-            {"response": "error", "error": "Parent commit out of date!"},
-            "test-uuid-1",
-            400,
-        )
         setup_resp_for_deleted_check(resp)
+        setup_resp_for_head_check(resp, commit_history[0])  # we are NOT on latest commit
 
-        with pytest.raises(GantryException):
-            gantry_dataset_obj.create_commit(COMMIT_MSG)
+        with pytest.raises(DatasetHeadOutOfDateException):
+            gantry_dataset_obj.commit(COMMIT_MSG)
 
         diff = gantry_dataset_obj.get_diff()
 
@@ -286,14 +418,16 @@ def test_create_commit_nochange(datadir, gantry_dataset_obj):
     """
     Test create commit without any local change
     """
-    with responses.RequestsMock() as resp:
-        setup_resp_for_deleted_check(resp, dataset_name="unittest_dataset")
+    dataset_name = "unittest_dataset"
 
-        gantry_dataset_obj.dataset_name = "unittest_dataset"
-        with open(datadir / "unittest_dataset/.dataset_metadata/HEAD") as f:
+    with responses.RequestsMock() as resp:
+        setup_resp_for_deleted_check(resp, dataset_name=dataset_name)
+
+        gantry_dataset_obj.dataset_name = dataset_name
+        with open(datadir / f"{dataset_name}/.dataset_metadata/HEAD") as f:
             old_commit = json.load(f)
 
-        new_commit = gantry_dataset_obj.create_commit("new commit")
+        new_commit = gantry_dataset_obj.commit("new commit")
         assert old_commit == new_commit
 
 
@@ -335,58 +469,9 @@ def test_sync_dataset(commit_id, datadir, gantry_dataset_obj, commit_history):
 
     with responses.RequestsMock() as resp:
         setup_resp_for_deleted_check(resp)
-        if commit_id is not None:
-            resp.add(
-                resp.GET,
-                f"{HOST}/api/v1/datasets/{DATASET_ID}/commits/{commit_id}",
-                json={
-                    "response": "ok",
-                    "data": commit_json,
-                },
-                headers={"Content-Type": "application/json"},
-            )
-        else:
-            resp.add(
-                resp.GET,
-                f"{HOST}/api/v1/datasets/{DATASET_ID}/commits",
-                json={
-                    "response": "ok",
-                    "data": [commit_json],
-                },
-                headers={"Content-Type": "application/json"},
-            )
+        setup_resp_for_sync(resp, commit_id, commit_json, manifest_file)
 
-        resp.add(
-            resp.POST,
-            f"{HOST}/api/v1/datasets/{DATASET_ID}/presign/getobject",
-            json={
-                "response": "ok",
-                "data": {
-                    MANIFEST_OBJ_KEY: f"{HOST}/fake_manifest",
-                    CONF_OBJ_KEY: f"{HOST}/fake_presigned_url",
-                    README_OBJ_KEY: f"{HOST}/fake_presigned_url",
-                },
-            },
-            headers={"Content-Type": "application/json"},
-        )
-
-        req_kwargs = {
-            "stream": True,
-        }
-        resp.add(
-            resp.GET,
-            f"{HOST}/fake_presigned_url",
-            body="fake_response",
-            match=[matchers.request_kwargs_matcher(req_kwargs)],
-        )
-
-        resp.add(
-            resp.GET,
-            f"{HOST}/fake_manifest",
-            body=manifest_file.read(),
-            match=[matchers.request_kwargs_matcher(req_kwargs)],
-        )
-        commit = gantry_dataset_obj.sync(commit_id)
+        commit = gantry_dataset_obj.pull(commit_id)
         assert commit == commit_json
         with open(datadir / "show_diff_dataset/dataset_config.yaml", "r") as f:
             assert f.read() == "fake_response"
@@ -454,48 +539,14 @@ def test_rollback(datadir, gantry_dataset_obj, commit_history):
         "parent_commit": new_commit["id"],
     }
 
+    # sync to local
     with responses.RequestsMock() as resp:
         setup_resp_for_deleted_check(resp)
-        resp.add(
-            resp.GET,
-            f"{HOST}/api/v1/datasets/{DATASET_ID}/commits",
-            json={
-                "response": "ok",
-                "data": commit_history,
-            },
-            headers={"Content-Type": "application/json"},
-        )
-        resp.add(
-            resp.POST,
-            f"{HOST}/api/v1/datasets/{DATASET_ID}/presign/getobject",
-            json={
-                "response": "ok",
-                "data": {
-                    MANIFEST_OBJ_KEY: f"{HOST}/fake_manifest",
-                    CONF_OBJ_KEY: f"{HOST}/fake_presigned_url",
-                    README_OBJ_KEY: f"{HOST}/fake_presigned_url",
-                },
-            },
-            headers={"Content-Type": "application/json"},
-        )
-        req_kwargs = {
-            "stream": True,
-        }
-        resp.add(
-            resp.GET,
-            f"{HOST}/fake_presigned_url",
-            body="fake_response",
-            match=[matchers.request_kwargs_matcher(req_kwargs)],
-        )
-        resp.add(
-            resp.GET,
-            f"{HOST}/fake_manifest",
-            body=manifest_file.read(),
-            match=[matchers.request_kwargs_matcher(req_kwargs)],
-        )
+        setup_resp_for_sync(resp, None, commit_history[0], manifest_file)
 
-        gantry_dataset_obj.sync()
+        gantry_dataset_obj.pull()
 
+    # try the rollback
     with responses.RequestsMock() as resp:
         setup_resp_for_deleted_check(resp)
         resp.add(
@@ -523,37 +574,12 @@ def test_rollback(datadir, gantry_dataset_obj, commit_history):
                 )
             ],
         )
-
-        resp.add(
-            resp.GET,
-            f"{HOST}/api/v1/datasets/{DATASET_ID}/commits/{rollback_commit['id']}",
-            json={
-                "response": "ok",
-                "data": rollback_commit,
-            },
-            headers={"Content-Type": "application/json"},
-        )
-        resp.add(
-            resp.POST,
-            f"{HOST}/api/v1/datasets/{DATASET_ID}/presign/getobject",
-            json={
-                "response": "ok",
-                "data": {
-                    MANIFEST_OBJ_KEY: f"{HOST}/fake_manifest",
-                    CONF_OBJ_KEY: f"{HOST}/fake_presigned_url",
-                    README_OBJ_KEY: f"{HOST}/fake_presigned_url",
-                },
-            },
-            headers={"Content-Type": "application/json"},
-        )
-        req_kwargs = {
-            "stream": True,
-        }
-        resp.add(
-            resp.GET,
-            f"{HOST}/fake_manifest",
-            body=manifest_file.read(),
-            match=[matchers.request_kwargs_matcher(req_kwargs)],
+        setup_resp_for_sync(
+            resp,
+            rollback_commit["id"],
+            rollback_commit,
+            manifest_file,
+            do_fake_presigned=False,
         )
 
         gantry_dataset_obj.rollback(original_commit["id"])
@@ -562,17 +588,16 @@ def test_rollback(datadir, gantry_dataset_obj, commit_history):
             assert json.load(f) == rollback_commit  # verify rollback has been successful
 
 
-@pytest.mark.parametrize("function", ["create_commit", "sync", "rollback"])
-def test_function_on_deleted_dataset(gantry_dataset_obj, function):
-
+@pytest.mark.parametrize("function", ["commit", "pull", "rollback"])
+def test_function_on_deleted_dataset(datadir, gantry_dataset_obj, function):
     with responses.RequestsMock() as resp:
         setup_resp_for_deleted_check(resp, is_deleted=True)
 
         with pytest.raises(GantryException):
-            if function == "create_commit":
-                gantry_dataset_obj.create_commit("fake commit message")
-            elif function == "sync":
-                gantry_dataset_obj.sync()
+            if function == "commit":
+                gantry_dataset_obj.commit("fake commit message")
+            elif function == "pull":
+                gantry_dataset_obj.pull()
             elif function == "rollback":
                 gantry_dataset_obj.rollback("eb2e7242-3340-4edf-8366-90d4fce897ce")
 
@@ -607,34 +632,11 @@ def test_add_file(
 
     with responses.RequestsMock() as resp:
         setup_resp_for_deleted_check(resp)
-        resp.add(
-            resp.POST,
-            f"{HOST}/api/v1/datasets/{DATASET_ID}/file",
-            json={
-                "response": "ok",
-                "data": {
-                    "created_at": "Wed, 05 Oct 2022 05:42:13 GMT",
-                    "created_by": "db459d6d-c83b-496d-b659-e48bca971156",
-                    "dataset_id": DATASET_ID,
-                    "id": "8ee0f6d5-c84c-473a-931a-5148b2e704d7",
-                    "is_latest_commit": True,
-                    "message": expected_commit_message,
-                    "metadata_s3_file_version": MANIFEST_VERSION_ID,
-                    "parent_commit": "eb2e7242-3340-4edf-8366-90d4fce897ce",
-                },
-            },
-            headers={"Content-Type": "application/json"},
-            match=[
-                matchers.multipart_matcher(
-                    {
-                        "file": (expected_filename, open(datadir / "test_file.txt", read_method)),
-                    },
-                    data={
-                        "email": "test@gantry.dev",
-                        "commit_msg": expected_commit_message,
-                    },
-                )
-            ],
+        setup_resp_for_add_file(
+            resp,
+            expected_commit_message,
+            expected_filename,
+            open(datadir / "test_file.txt", read_method),
         )
 
         gantry_dataset_obj.add_file(
@@ -663,34 +665,11 @@ def test_add_tabular_file(
 
     with responses.RequestsMock() as resp:
         setup_resp_for_deleted_check(resp)
-        resp.add(
-            resp.POST,
-            f"{HOST}/api/v1/datasets/{DATASET_ID}/file",
-            json={
-                "response": "ok",
-                "data": {
-                    "created_at": "Wed, 05 Oct 2022 05:42:13 GMT",
-                    "created_by": "db459d6d-c83b-496d-b659-e48bca971156",
-                    "dataset_id": DATASET_ID,
-                    "id": "8ee0f6d5-c84c-473a-931a-5148b2e704d7",
-                    "is_latest_commit": True,
-                    "message": expected_commit_message,
-                    "metadata_s3_file_version": MANIFEST_VERSION_ID,
-                    "parent_commit": "eb2e7242-3340-4edf-8366-90d4fce897ce",
-                },
-            },
-            headers={"Content-Type": "application/json"},
-            match=[
-                matchers.multipart_matcher(
-                    {
-                        "file": (expected_filename, csvfile),
-                    },
-                    data={
-                        "email": "test@gantry.dev",
-                        "commit_msg": expected_commit_message,
-                    },
-                )
-            ],
+        setup_resp_for_add_file(
+            resp,
+            expected_commit_message,
+            expected_filename,
+            csvfile,
         )
 
         csvfile.seek(0)
@@ -728,42 +707,18 @@ def test_add_tabular_file_bad_name(gantry_dataset_obj, real_filename, provided_f
 def test_add_dataframe(gantry_dataset_obj, filename, expected_filename, commit_message):
     expected_commit_message = commit_message or f"Uploaded {expected_filename} to dataset"
 
+    df = pd.DataFrame({"A": [100, 101], "B": [200, 202]})
     expected_file = StringIO()
-    pd.DataFrame({"A": [100, 101], "B": [200, 202]}).to_csv(expected_file)
+    df.to_csv(expected_file)
     expected_file.seek(0)
 
     with responses.RequestsMock() as resp:
         setup_resp_for_deleted_check(resp)
-        resp.add(
-            resp.POST,
-            f"{HOST}/api/v1/datasets/{DATASET_ID}/file",
-            json={
-                "response": "ok",
-                "data": {
-                    "created_at": "Wed, 05 Oct 2022 05:42:13 GMT",
-                    "created_by": "db459d6d-c83b-496d-b659-e48bca971156",
-                    "dataset_id": DATASET_ID,
-                    "id": "8ee0f6d5-c84c-473a-931a-5148b2e704d7",
-                    "is_latest_commit": True,
-                    "message": expected_commit_message,
-                    "metadata_s3_file_version": MANIFEST_VERSION_ID,
-                    "parent_commit": "eb2e7242-3340-4edf-8366-90d4fce897ce",
-                },
-            },
-            headers={"Content-Type": "application/json"},
-            match=[
-                matchers.multipart_matcher(
-                    {
-                        "file": (expected_filename, expected_file),
-                    },
-                    data={
-                        "email": "test@gantry.dev",
-                        "commit_msg": expected_commit_message,
-                    },
-                )
-            ],
+        setup_resp_for_add_file(
+            resp,
+            expected_commit_message,
+            expected_filename,
+            expected_file,
         )
 
-        gantry_dataset_obj.add_dataframe(
-            pd.DataFrame({"A": [100, 101], "B": [200, 202]}), filename, commit_message
-        )
+        gantry_dataset_obj.add_dataframe(df, filename, commit_message)
