@@ -1322,6 +1322,7 @@ def test_log_records_preds_and_feedback_series_coerced(
         {"feedback_keys": ["A"]},
         {"feedback_ids": ["12345", "67890"]},
         {"join_keys": ["54321", "67890"]},
+        {"join_keys": pd.Series(["54321", "67890"])},
         {},
     ],
 )
@@ -1460,12 +1461,43 @@ def test_log_records_preds_and_feedback_wrong_np_array(
     mock_feedback.assert_not_called()
 
 
+@pytest.mark.parametrize("join_keys", [[1, 2, 3], pd.Series([1, 2, 3])])
+@mock.patch("gantry.logger.client.Gantry._log_prediction_and_feedback_events")
+@mock.patch("gantry.logger.client.Gantry._log_prediction_events")
+@mock.patch("gantry.logger.client.Gantry._log_feedback_events")
+def test_log_records_preds_and_feedback_wrong_join_keys(
+    mock_feedback,
+    mock_preds,
+    mock_preds_and_feedback,
+    join_keys,
+    cli_obj,
+):
+    assert (
+        cli_obj.log_records(
+            application="foobar",
+            version="1.2.3",
+            inputs=[{"A": 200}, {"A": 201}],
+            outputs=[{"B": 200}, {"B": 201}],
+            feedbacks=[{"C": 200}, {"C": 201}],
+            timestamps=None,
+            sort_on_timestamp=True,
+            sample_rate=1,
+            join_keys=join_keys,
+        )
+        is None
+    )
+    mock_preds_and_feedback.assert_not_called()
+    mock_preds.assert_not_called()
+    mock_feedback.assert_not_called()
+
+
 @pytest.mark.parametrize(
     "kwargs",
     [
         {"feedback_keys": ["A"]},
         {"feedback_ids": ["12345", "67890"]},
         {"join_keys": ["54321", "67890"]},
+        {"join_keys": pd.Series(["54321", "67890"])},
         {},
     ],
 )
@@ -1559,6 +1591,7 @@ def test_log_records_preds_only(
         {"feedback_keys": ["A"]},
         {"feedback_ids": ["12345", "67890"]},
         {"join_keys": ["54321", "67890"]},
+        {"join_keys": pd.Series(["54321", "67890"])},
     ],
 )
 @pytest.mark.parametrize(
